#include <stdlib.h>
#include <stream.h>
#include <string.h>

/*
 * It's time to update and print the tourist information we have for New
 * England and the Pacific Northwest.  This program first prints and
 * optionally modifies the tourist information for each state, then creates 
 * a database that is a guide to Northern US seacoasts.
 */

DATABASE atlas = FILENAME "atlas.gdb";
DATABASE guide = FILENAME "c_guide.gdb";

#include <stdio.h>

static char *state_list[] =
	{"ME", "NH", "MA", "RI", "CT", "OR", "WA", "AK", 0};

static void clean_up (char** states_list);
static void transfer_data (char** states_list);
static int get_string (char* prompt_string, char *response_string, int length);

#define GET_STRING(p,b)	get_string (p, b, sizeof (b))
#define FALSE 0
#define TRUE 1

main ()
{
/**************************************
 *
 *   m a i n
 *
 **************************************
 *
 * Functional description
 *
 * The main routine establishes a list of
 * states from the main atlas database.  Its
 * subroutines allow the user to update guidebook 
 * descriptions of those states, and store them
 * in a separate database.
 **************************************/


READY atlas;
READY "nc_guide.gdb" AS guide;

clean_up (state_list);
transfer_data (state_list);

FINISH atlas;
FINISH guide;
exit (0);
}

static void clean_up (char** states_list)
{
/**************************************
 *
 *   c l e a n _ u p
 *
 **************************************
 *
 * Functional description
 *
 * At the users' choice, edit the tourism blob for selected states,
 * load a new one from a named file, or accept text from standard input
 **************************************/

char	filename[40], choice[4], *state;
BSTREAM	*tour;
gds_tr_handle update_tr;
int	c;

/* start a named transaction for update, reserving the necessary relations */

update_tr = 0;
START_TRANSACTION update_tr CONCURRENCY READ_WRITE 
	RESERVING atlas.tourism FOR WRITE,
	atlas.states FOR READ;

for (state = *states_list++; state; state = *states_list++ )
	{                                                                    
	FOR (TRANSACTION_HANDLE update_tr) s IN atlas.STATES 
		CROSS t IN atlas.TOURISM OVER state WITH s.state = state
		{
		cout << "Here's the information on " << s.state_name << ".\n\n" << flush;
		BLOB_display (&t.guidebook, atlas, update_tr, "guide_book");
		GET_STRING ("\nIs this information up to date? (Y/N) ", choice);
		if (*choice != 'y' && *choice != 'Y')
		MODIFY t USING
			{
 			GET_STRING ("\nEnter F to update from a file, E to edit the description ",
				choice); 
			if (*choice == 'F' || *choice == 'f')
				{
				GET_STRING ("Enter full path name of input file ", filename);
				BLOB_load (&t.guidebook, atlas, update_tr, filename);
				}
			else if (*choice == 'E' || *choice == 'e')
				BLOB_edit (&t.guidebook, atlas, update_tr, "guidebook");
			else
				{
				cout << "Enter new information from standard input.\n";
          			cout << "Terminate with <EOF>.\n";
          			if (tour = Bopen (&t.guidebook, atlas, update_tr, "w"))
					while ((c = cin.get ()) != EOF)
						putb (c, tour);
				cin.clear(0);
				BLOB_close (tour);
          			}
			}
		END_MODIFY;
		}
	END_FOR;
	}
COMMIT update_tr;
}


static int get_string (char* prompt_string, char *response_string, int length)
{
/**************************************
 *
 *	g e t _ s t r i n g
 *
 **************************************
 *
 * Functional description
 *	Write a prompt and read a string.  If the string overflows,
 *	try again.  If we get an EOF, return FAILURE.
 *
 **************************************/
char	*p;
int	l, c;

while (1)
    {
    cout << prompt_string << ": ";
    for (p = response_string, l = length; l; --l)
	{
	c = cin.get ();
	if (c == EOF)
	    return FALSE;
	if (c == '\n')
	    {
	    *p = 0;
	    return TRUE;
	    }
	*p++ = (char) c;
	}
    while (cin.get () != '\n')
	;
    cout << "** Maximum field length is " << length - 1 << " characters **\n";
    }
}

static void transfer_data (char** states_list)
{
/**************************************
 *
 *   t r a n s f e r _ d a t a
 *
 **************************************
 *
 * Functional description
 *
 * Move the tourism information for selected
 * states to the guide_book database.
 **************************************/

gds_tr_handle trans_tr;
int	c;
BSTREAM	*b_in, *b_out;
char	*state;

trans_tr = 0;

START_TRANSACTION trans_tr CONCURRENCY READ_WRITE
	RESERVING atlas.tourism FOR READ,
	guide.tourism FOR WRITE;

for (state = *states_list++; state; state = *states_list++ )
	{                                                                    
	FOR (TRANSACTION_HANDLE trans_tr) s IN atlas.STATES 
		CROSS t IN atlas.TOURISM OVER state WITH s.state = state
		{
		STORE (TRANSACTION_HANDLE trans_tr) new IN guide.TOURISM USING
			strcpy (new.state_name, s.state_name);
			strcpy (new.state, s.state);
			strcpy (new.city, t.city);
          		strcpy (new.zip, t.zip); 

			/* One way to copy a blob */
			CREATE_BLOB n_addr IN new.office;
			FOR o_addr IN t.office
				strncpy (n_addr.SEGMENT, o_addr.SEGMENT, sizeof(n_addr.SEGMENT));
				n_addr.LENGTH = o_addr.LENGTH;
				PUT_SEGMENT n_addr;
			END_FOR;
			CLOSE_BLOB n_addr;

			/* another way to copy a blob */
			b_in = Bopen (&t.guidebook, atlas, trans_tr, "r");
			b_out = Bopen (&new.guidebook, guide, trans_tr, "w");
			while ((c = getb (b_in)) != EOF)
	   			putb (c, b_out);
    	 		BLOB_close (b_in);
   	   		BLOB_close (b_out);
		END_STORE;
 		}
	END_FOR;
	}
PREPARE trans_tr
	ON_ERROR
		{
		cout << "Error preparing a multi_database transaction\n";
		cout << "Please manually rollback limbo transactions in GUIDE and ATLAS\n";
		ROLLBACK trans_tr;
		return;
		}
	END_ERROR;

COMMIT trans_tr
	ON_ERROR
		{
		cout << "Error committing a prepared multi_database transaction\n";
		cout << "Please manually commit limbo transactions in GUIDE and ATLAS\n";
		}
	END_ERROR;
}








                                                                             
 
